%   SOFTWARE BUNDLE for the CONDITIONAL SIMULATION of Gaussian random 
%   fields with Spartan covariance function as of APRIL 5, 2017.
%   
%   RUN_CONDSIM_SSRF_v1 generates a sample drawn from a Gaussian random field
%   with Spartan covariance function. This is used as the dataset. 
%   Then, a number of conditional simulations are generated based on the
%   method that combines unconditional simulations and kriged estimates of
%   the data according to the following publication: 
%
%   Davis, M. W. (1987). Production of conditional simulations via the LU 
%   triangular decomposition of the covariance matrix. 
%   Mathematical Geology, 19(2), 91-98.
%
%   WARNING: In the following comments, the names of variables, functions 
%   and parameters are capitalized. This does not reflect the actual
%   capitalization of the names. 
%
%================ INPUT PARAMETERS ========================================
%
%   N:          Number of sampling sites
%   NSIM:       Number of simulations
%   L:          Length of bounding box
%   MX:         Mean value of random field
%   PARAM:      Values of the SSRF parameters eta0, eta1, xi
%
%================ OUTPUT PARAMETERS =======================================
%
%   PROGRAM OUTPUTS INCLUDE THE FOLLOWING
%
%   ZCSIM:      Array containing the simulated vales. Size L x L x NSIM
%   ZSIMU:      Array containing the simulated vales. Size L x L x NSIM
%
%==========================================================================
%
%  Copyright (C) 2017  Dionisis Hristopulos
%   email: dionisi@mred.tuc.gr
%
%  This program is free software; you can redistribute it and/or modify it
%  under the terms of the GNU General Public License as published by the
%  Free Software Foundation (version 2 of the License or  later version).
%
%  This program is distributed WITHOUT ANY WARRANTY.
%  See the GNU General Public License for more details.
%
%   This software bundle also includes code developed by other people:
%   kNearestNeighbors, smplot
%   This bundle calls the following functions: 
%   COV_SSRF_EXP, KRIG_SSRF, SPARTAN_SIM, SIMUL_FACTOR
%==========================================================================

clear vars; close all; 

%===========INITIALIZATION OF PARAMETERS ==================================
Param = [10, 5, 3]; 
N = 150;    %   Number of sampling sites
%K = 8;    %	Number of realizations
L = 50;    %   Length of bounding box
Nsim = 12;  %	Number of simulations
mx = 5;     %   Mean value of the field
pos = L * rand(N, 2); 

%==========================================================================

ds = pdist(pos);

ds = squareform(ds);

%   Generate Spatan covariance matrix
[~, covar] = cov_ssrf_exp(2, ds, Param, 0);

%   Generate realization of Gaussian random field with Spartan covariance
%   These become the data 
xs = pos(:,1);
ys = pos(:,2);

zs = mx + simul_factor_v1(pos, 0, 'Spartan', Param, 'sqrt', 1);

% Map grid definition

lv = 1:1:L; 
[xx,yy] = meshgrid(lv, lv);

xg = reshape(xx, L*L,1);
yg = reshape(yy, L*L,1);

%   Find nearest neighbor grid sites to the sampling sites

[nIds, nDist] = kNearestNeighbors([xg yg], [xs ys], 1);

%   Find grid sites that will be omitted from the interpolation 
%   Vector IN  contains an index to these sites

zest = nan*ones(size(xg)); 

TRI = delaunay(xs,ys);
xg=xg(:); yg=yg(:); 

% Find the nearest triangle (Tnb)
Tnb = tsearchn([xs,ys], TRI, [xg,yg]);
% Only keep the relevant triangles.
in= find(~isnan(Tnb));

size(in), 

if isempty(TRI),
  	warning('Data cannot be triangulated.');
  	return
end

%   Plot sampling sites and nearest map grid sizes; 

figure(1); 
plot(xg(in), yg(in), 'g.'); % Plot grid sites
hold on;  
plot(xs, ys, 'o');  % plot sampling sites
hold on; 
plot(xg(nIds), yg(nIds),'k+'); % plot grid sites nearest to sampling sites
axis tight;
axis equal; 

zs = zs(:); 

Paramd = [0 Param]; % Covariance model parameters with zero nugget

%   Z^{c} = Z^{u} + \hat{Z} - \hat{Z^{u}

% Perform SSRF spatial interpolation of  fluctuations on map grid using
% sample values and ordinary kriging to generate \hat{Z}

[hatz, ~, ~, ~, ~] = krig_ssrf_v1(pos, zs, [xg(:), yg(:)], Paramd);
hatz = reshape(hatz, L, L); 

%   Plot kriged data
figure(2); 
imagesc(hatz);
title('Kriged data'); 
axis tight; 
axis equal; 
colorbar;
colormap('cool'); 

hold on; 
plot(xs, ys, 'k.'); 

tic; 

Zcsim = zeros(L, L, Nsim); 
Zsimu = zeros(L, L, Nsim); 

for i=1:Nsim
    
    %   Perform unconditional simulations of normalized fluctuations

    [~, ~, zu] =  spartan_sim_v1(L, L, Param);
    zu = zu + mx; 
    
    % Generate unconditional realization at the field sites using
    % approximate values
    
%    zud = mvnrnd(mx*ones(1,N), covar, 1); 
    zud = zu(nIds);
    zud = zud(:); 

    % Generate kriged "data values" from the unconditional realization at 
    % the data points  
    
    [hatzu, ~, ~, ~, ~] = krig_ssrf_v1(pos, zud, [xg(:), yg(:)], Paramd);
    hatzu = reshape(hatzu, L, L); 
    
    %   Z^{c} = Z^{u} + \hat{Z} - \hat{Z^{u}    
    %   Calculate conditional simulation of the normalized fluctuations
    
    zcsim = zu +  hatz - hatzu; 
               
    %   Vectors containing the conditional and unconditional simulated
    %   values at the map grid nodes
    Zcsim(:, :, i) = zcsim; 
    Zsimu(:, :, i) = zu;
    
    zmin(i) = min(min(zcsim)); 
    zmax(i) = max(max(zcsim));  
    
    zumin(i) = min(min(zu)); 
    zumax(i) = max(max(zu));  
        
end; 

zminmin=min(zmin); 
zmaxmax=max(zmax); 

zminminu=min(zumin); 
zmaxmaxu=max(zumax); 


if Nsim <= 12
    %   FIGURE 10: Plots of conditional simulations
    
    figure(10);
    for i=1:Nsim
        
        zreal = Zcsim(:, :, i);
        
        smplot(Nsim/4, 4, i,'right',0.1,'axis','on')
        imagesc(Zcsim(:, :, i));
        shading flat
        caxis([zminmin zmaxmax])
        axis off
        
    end;
    
    cb = colorbar('east');
    set(cb,'Position',[0.92 0.1 0.04 0.8]);
    title('Conditional Simulations');
    
     
    %   FIGURE 12: Plots of unconditional simulations
    figure(12);
    
    for i=1:Nsim
        
        zrealu = Zsimu(:, :, i);
        
        smplot(Nsim/4, 4, i,'right',0.1,'axis','on');
        imagesc(Zsimu(:, :, i));
        shading flat
        caxis([zminminu zmaxmaxu])
        axis off
    end
    cbu = colorbar('east');
    set(cbu,'Position',[0.92 0.1 0.04 0.8])
    title('Unconditional Simulations');
        
end;

figure(13); 
%   Calculate the mean and variance of the conditional simulations
meanz = mean(Zcsim,3);
varz = var(Zcsim,0,3); 
subplot(2,1,1); 
meanz = reshape(meanz, L, L); 
imagesc(meanz); 
title('Ensemble Mean');
colormap('cool');  
%colormap(b2r(min(min(meanz)), max(max(meanz))));
axis tight; axis equal;
colorbar; 

subplot(2,1,2); 
stdz=sqrt(varz); 
stdz=reshape(stdz, L, L);
imagesc(stdz); 
title('Ensemble St. Dev.');
colormap('cool');  

axis tight; axis equal;
colorbar; 

%==========================================================================

time=toc/Nsim; 

return;