function [h,covar]=cov_ssrf_exp(dd,lag,Param,flag)
%  COV_SSRF_EXP calculates the explicit SSRF covariance function at the
%   limit of an infinite spectral cutoff. The possibility of a nugger term
%   is also included via a fourth (optional) parameter in the vector Param
%
% This routine was tested under Matlab R2011b 7.13.0.564 32-bit (win32)
%
%==========================================================================
%   Input parameters
%==========================================================================
%
%   dd:       Dimensionality (dd=1,2,3)
%	lag:       Lag vector
%	Param:     Spartan parameters [e0,e1,xi, nugget]
%   flag:     If flag=0 the plots are suppressed
%
%==========================================================================
%   Output parameters
%==========================================================================
%
%	h       Normalized (by xi) lag vector
%	covar     Covariance vector
%
%	EXAMPLE
%==========================================================================
%
%	[h,covar]=cov_ssrf_exp(2, 0:1:10, [1,2,3], 1);
%
%  Copyright (C) 2016 Dionisis Hristopulos, 
%   email: dionisi@mred.tuc.gr
%
%  This program is free software; you can redistribute it and/or modify it
%  under the terms of the GNU General Public License as published by the
%  Free Software Foundation (version 2 of the License or  later version).
%
%  This program is distributed WITHOUT ANY WARRANTY.
%  See the GNU General Public License for more details.
%==========================================================================
%	REFERENCES
%==========================================================================
%
%   D. T. Hristopulos, Spartan Gibbs random field models for 
%   geostatistical applications, SIAM Journal of Scientific Computing,
%   24 (6), pp. 2125-2162 (2003).
%
%   D. T. Hristopulos and S. Elogne, Analytic properties and covariance 
%   functions for a new class of generalized Gibbs random fields, 
%   IEEE Transactions on Information Theory, 53(12), 4467-4679 (2007).
%
%   D. T. Hristopulos. Covariance functions motivated by spatial random 
%   field models with local interactions. Stochastic Environmental Research
%   and Risk Assessment, 29(3):739754 (2015)
%
%==========================================================================

e0=Param(1);
e1=Param(2);
xi=Param(3);
nparam=length(Param);


%   Calculate the normalized lag
h=lag/xi;

%    Replace zero lag to calculate the De l'Hospital limit
h(lag==0)=eps;

switch dd
    
    case 1
        
        if abs(e1)<2
            
            be1 = 0.5*(sqrt(2-e1));  
            be2 = 0.5*(sqrt(2+e1));
            Gr = (exp(-h*be2)).*( (cos(h*be1))/be2 +(sin(h*be1))/be1)/4;
        
        elseif  e1==2
            
            Gr = ((1+h).*exp(-h))/4;
            
        elseif e1>2
            
            delta = sqrt(e1^2-4); 
            w2 = sqrt((e1+delta)/2);
            w1 = sqrt((e1-delta)/2);
            Gr = ( (exp(-h*w1))/w1 - (exp(-h*w2))/w2 )/(2*delta);
        
        else
            disp('Permissibility conditions fail');
        end
        
    case 2
        
        if e1==2
            Gr = h.* besselk(-1,h)/(4*pi);
            vv = 1/(4*pi);
        
        elseif e1 >2
            
            Dn = sqrt(e1^2-4);
            zp = sqrt(- (- e1 + Dn)/2);
            zm = sqrt(- (- e1 - Dn)/2);
            Gr = ( besselk(0,h*zp)-besselk(0,h*zm))/(2*pi*Dn );
            vv =  log( (e1+Dn)/(e1-Dn))/ (4 *pi *Dn);
        
        elseif abs(e1)<2
            
            Dn = sqrt(4-e1^2);
            zp = sqrt(- (- e1 + sqrt(-1)*Dn)/2);
            zm = sqrt( - (- e1 - sqrt(-1)*Dn)/2);
            Gr = imag(besselk(0,h*zp))/ ( pi * Dn);
            vv = (pi/2 - atan(e1/Dn)) / (2 *pi *Dn);
            
        else
            disp('Permissibility conditions fail');
            
        end
        
    case 3
        
        if abs(e1)<2
            
            delta = sqrt(4-e1^2);
            be1 = 0.5*(sqrt(2-e1)); 
            be2 = 0.5*(sqrt(2+e1));
            Gr = (exp(-h*be2)).*(sin(h*be1)./(2*pi*h*delta));
            
        elseif  e1==2
            
            Gr=exp(-h)/(8*pi);
            
        elseif e1>2
            
            delta = sqrt(e1^2-4); 
            w2 = sqrt((e1+delta)/2);
            w1 = sqrt((e1-delta)/2);
            Gr = (exp(-h*w1)-exp(-h*w2))./(4*pi*h*delta);
            
        else
            disp('Permissibility conditions fail');
            
        end
        
    otherwise
        disp('Not defined for d>3');
        
end

if e1 <= -2 
    covar=nan(size(h));
    
else
    if nparam==4
        %C0=max(Param(4),0);
        C0 = Param(4);
        covar = e0*Gr + C0;
    else
        covar=e0*Gr;
    end
    
    if nargin >3 && flag>0
        figure(1);
        plot(h,covar,'b-');
        if length(h)<20
            hold on; 
            plot(h,covar,'r.');
        end;
        xlabel('h'); 
        ylabel('SRRF Covariance');
        title(['d=',num2str(dd)]);
        axis tight; 
        %figbolden
    end;
end
