% FUNCTION KRIG_SSRF performs Simple/Ordinar/Trend Kriging. The function is
% based on the mGStat function. 
%
%================ INPUT PARAMETERS ========================================
%
%   POS_KNOWN:     Array containing the ampling sites
%   Z_KNOWN:       Array containing the sample values
%   POS_EST:       Array containing the prediction sites
%   PARAMD:        Parameters of the SSRF model
%   OPTIONS:      Values of the SSRF parameters eta0, eta1, xi
%
%================ OUTPUT PARAMETERS =======================================
%
%   PROGRAM OUTPUTS INCLUDE THE FOLLOWING
%
%   Z_EST:      Array containing the simulated vales. Size L x L x NSIM
%   KRIG_VAR:   Array containing the simulated vales. Size L x L x NSIM
%   LAMBDA:     Vector of kriging weights
%   CXX:        Sampling locations covariance matrix
%   CZ:         Covariancebetween sampling sites and prediction points
%   OPTIONS:    Optional argument used to determine type of kriging method
%               DEFAULT: Ordinary Kriging
%               MEAN OR SK_MEAN: Simple kriging
%               POLYTREND:       Kriging with trend
%
%==========================================================================
%
% EXAMPLE:
% [Z_est, Krig_var, lambda, Cxx, Cz]= krig_ssrf_v1(pos_known, Z_known, pos_est,Paramd,options);
%
%==========================================================================

function [Z_est, Krig_var, lambda, Cxx, Cz] = krig_ssrf_v1(pos_known, ...
                        Z_known, pos_est, Paramd, options)


Nugget = Paramd( 1);

Param = Paramd(2:length(Paramd));

if nargin<5
    options.null=1;
end

% DETERMINE TYPE OF KRIGING
if  any(strcmp(fieldnames(options),'polytrend'))
    ktype=2; % Ktrend
    mgstat_verbose('Kriging with a trend',20)
elseif  (any(strcmp(fieldnames(options),'mean')) || ...
                    any(strcmp(fieldnames(options),'sk_mean')))
    ktype=0; % SK
    mgstat_verbose('Simple Kriging',20)
else
    ktype=1; % OK
    mgstat_verbose('Ordinary Kriging',20)
end

N = size(pos_known,1);
ndim = size(pos_known,2);
Nest = size(pos_est,1);

Zmean = mean(Z_known);

%   Determine Kriging Method
if ktype==0
    Cxx = zeros(N,N);
    Cz =zeros(N,Nest);
elseif ktype==1
    Cxx = zeros(N+1,N+1);
    Cz =zeros(N+1,Nest);
else % KTREND
    Cxx = zeros(N+ndim,N+ndim);
    Cz = zeros(N+ndim,Nest);
end

%   Determine matrix of data to data distances

if N <=30, opt=1;
elseif (30< N) && (N<=300), opt=2;
else opt=3;
end

switch ndim
    case rem(ndim,1)~=0
        disp('Non-integer dimension');
        return;
    case ndim<0
        disp('Negative dimension');
        return;
    case 2
        rxx = distmat(pos_known,opt);
    case 1
        xs=pos_known;
        rxx = abs(xs*ones(1,N)-ones(N,1)*xs');
    otherwise
        rxx = distmat(pos_known,opt);
end;

%figure; surf(rxx); shading flat; view(2) ; pause;


rxz = pdist2(pos_est,pos_known)';% This is an N x Nest matrix

idupl = find(rxz ==0);

%   Calculate Kriging covariance matrices

[~,Cxx0] = cov_ssrf_exp(ndim,rxx,Param,0);

Cxx0 = Cxx0 + Nugget * diag(ones(1,N)) ;

[~, Cz0] = cov_ssrf_exp(ndim,rxz,Param,0);

if ~isempty((idupl))
    
    Cz0(idupl) = Cz0(idupl) + Nugget ;
end

%figure; surf(Cxx0); shading flat; view(2) ; pause;

% ADJUST Cxx and k for KRIGING METHODS
if ktype==1
    
    Cxx(1:N,1:N) = Cxx0;
    Cxx(N+1,1:N)=ones(1,N);
    Cxx(1:N,N+1)=ones(N,1);
  
    Cz(1:N,:) = Cz0;
    Cz(N+1,:)=1;
    
    clear Cz0 Cxx0;
    
elseif ktype==2
    
    if isfield(options,'polytrend')==0
        polytrend=1;
    else
        polytrend=options.polytrend;
    end
    
    if length(polytrend)==1,
        polytrend=ones(1,ndim).*polytrend;
    end
    
    %polytrend=4;
    
    Cxx(1:N,1:N) = Cxx0;
    Cxx(N+1,1:N)=ones(1,N);
    
    Cxx(N+1,1:N)=ones(size(pos_known(:,1),1),1);
    Cxx(1:N,N+1)=ones(size(pos_known(:,1),1),1);
    
    Cz(1:N,:) = Cz0;
    Cz(N+1,:)=1;
    
    clear Cz0 Cxx0;
    
    ik=1;
    for id=1:ndim
        for it=1:1:polytrend(id)
            ik=ik+1;
            Cxx(N+ik,1:N) = [pos_known(:,id)'].^(it);
            Cxx(1:N,N+ik) = [pos_known(:,id)].^(it);
            Cz(N+ik,:) = [pos_est(id)].^(it);
        end
    end
    
else
    Cxx(1:N,1:N) = Cxx0;
    
    Cz(1:N,:) = Cz0;
    clear Cz0 Cxx0;
end

if any(strcmp(fieldnames(options),'trend'));
    % krig only the trend, by setting data to uN covariances
    % to zero
    Cz(1:N,:)=0;
end

Z_est = zeros(Nest,1);
Krig_var = zeros(Nest,1);
lambda = zeros(length(Cxx),Nest);

for i = 1: Nest
    
    Czv = Cz(:,i);
    
    % SOLVE THE LINEAR SYSTEM
    
    lambda(:,i) = Cxx\Czv;
    
    if ktype == 0
        Z_est(i) = (Z_known' - Zmean)* lambda(:,i) + Zmean ;
    elseif ktype == 1
        Z_est(i) =  Z_known' * lambda(1:N,i);
    elseif ktype == 2
        Z_est(i) = Z_known' * lambda(1:N,i);
    else
    end
    Krig_var(i) = Cxx(1,1) - Czv' * lambda(:,i);
    
end
