function rf = simul_factor_v1(locs, fm, model, Param, method, rngo)
%  SIMUL_FACTOR generates random realizations of a random field using
%  matrix factorization methods. 
%
% This routine was tested under Matlab R2011b 7.13.0.564 32-bit (win32)
%
%==========================================================================
%   Input parameters
%==========================================================================
%
%   locs:       Locations of simulated field (N x d) array where d space
%               dimension
%	fm:         Mean value
%   model:      Covariance model ('Gaus', 'Expo', 'Spartan', 'Matern')      
%	Param:      Covariance parameters
%               Exponential paramters & Gaussian [varx xi]
%               Spartan parameters [e0,e1,xi]
%               Matern parameters [varx xi nu]
%   method:     Factorization method used (Cholesky, Principal Square Root, 
%               LU decomposition
%   rngo:       Optional control for random number generator. If rngo is
%               not empty, the random number generator uses its default 
%               settings
%
%==========================================================================
%   Output parameters
%==========================================================================
%
%	rf:         Random field (array N x 1)
%
%	EXAMPLE
%==========================================================================
%
%	rf = simul_factor_v1(locs, 0, 'Spartan', [100 1.5 4], 'sqrt', 1);
%
%==========================================================================
%  Copyright (C) 2017 Dionisis Hristopulos, 
%   email: dionisi@mred.tuc.gr
%
%  This program is free software; you can redistribute it and/or modify it
%  under the terms of the GNU General Public License as published by the
%  Free Software Foundation (version 2 of the License or  later version).
%
%  This program is distributed WITHOUT ANY WARRANTY.
%  See the GNU General Public License for more details.
%==========================================================================
%	REFERENCES for the Spartan covariance function
%==========================================================================
%
%   D. T. Hristopulos, Spartan Gibbs random field models for 
%   geostatistical applications, SIAM Journal of Scientific Computing,
%   24 (6), pp. 2125-2162 (2003).
%
%   D. T. Hristopulos and S. Elogne, Analytic properties and covariance 
%   functions for a new class of generalized Gibbs random fields, 
%   IEEE Transactions on Information The0ry, 53(12), 4467-4679 (2007).
%
%   D. T. Hristopulos. Covariance functions motivated by spatial random 
%   field models with local interactions. Stochastic Environmental Research
%   and Risk Assessment, 29(3):739754 (2015)
%
%==========================================================================
 r = pdist(locs); 
 
 dd = size(locs,2); % Data dimension
 
 N = length(locs);

 %  Construct covariance matrix for different covariance models
 
if strcmp(model,'Gaus')
   ts = 'Gaussian'; 
   
   varf = Param(1); 
   xi = Param(2); 
   C = varf * exp(- (r./xi).^2 );

elseif	strcmp(model,'Expo')
   ts = 'Exponential';
   
   varf = Param(1); 
   xi = Param(2); 
   C = varf * exp(- (r./xi) );
   
elseif strcmp(model,'Spartan')
   ts = 'Spartan';
   
   [~, C] = cov_ssrf_exp(dd, r, Param, 0);
   
elseif strcmp(model,'Matern')
    
   varf = Param(1); 
   xi = Param(2); 
   nu = Param(3); 
   
   Hr = r/xi;
   r1 = (2^(nu-1))*gamma(nu);
   bes = besselk(nu, Hr);
   i=find(isinf(bes));
   if ~isempty(i)
       bes(i)= besselk(nu, eps); 
       r1(i) = eps; 
   end
   C = (varf./r1).*(Hr.^nu) .* bes;
     
else
	sprintf('%s','Unrecognizable Model')
	pause;
end;

%   Put the covariance matrix in square form

C = squareform(C); 

%   Covariance matrix decomposition

switch method
    case 'sqrt'
        A = real(sqrtm(C));
    case 'chol'
        A = chol(C);
    case 'lu'
        A = lu(C);
end

%   Consruction of random field

if nargin==6 && ~isempty(rngo)
    rng('default');
end
rf = fm + A * randn(N,1); 


