%   SPARTAN_SIM zero-mean, isotropic 2D, Spartan spatial random field 
%   realization using a spectral simulation method. 
%	It is based on a Spartan Gibbs probability density
%	with local, gradient and Laplacian coupling, i.e.,
%
%	P(X)~ exp[-H/(2* eta0* xi^d)], where
%	H= Int(X^2) + eta1*xi^2 * Int(GRAD(X)^2) + xi^4 * Int(Lapl(X)^2)
%
%	[x,y,rf]=spartan_sim_v1(Lx,Ly,[eta0,eta1,xi],[],[])
%
%	The field is generated with the Fourier Filtering Method (FFM)
%	that uses efficiently the FFT2 fuction.
%
%==========================================================================
%	INPUT VARIABLES:
%==========================================================================
%
%	Lx:		Length of the domain in the x-direction (integer)
%	Ly:		Length of the domain in the y-direction (integer)
%   Param:  Vector of four parameters: [eta0 eta1 xi km]
%	eta0:		Scaling term
%	eta1:		Coupling strength of [grad(X)]^2 term
%	xi:		Length scale (proportional to correlation length)
%   km:     Optional spatial frequency cutoff
%
%======= OPTIONAL==========================================================
%
%   sd:     Large integer used to seed the random number generator
%   (optional). If sd is missing, the random number generator is seeded
%   according to clock
%   ifig:    Any value for plotting the simulated state
%
%   REMARK: This function has not been adequately tested for rectangular
%   grids and for grid spacing different than one. 
%
%   FOR FURTHER EXTENSION
%
%	AR:		Anisotropic ratio AR=bx/by (real number > 1)
%	THETA:	Angle of principal directions with respect to
%			coordinate system
%
%==========================================================================
%	OUTPUT VARIABLES:
%==========================================================================
%
%	[x,y]:	Numerical grid
%	rf:		Spartan Random field values at the grid nodes
%
%==========================================================================
%	EXAMPLE:
%
%==========================================================================
%	[x,y,rf] = spartan_sim(100,100,[1 0.2 5],123456789,[]);
%
%==========================================================================
%  Copyright (C) 2013 Dionisis Hristopulos, 
%   email: dionisi@mred.tuc.gr
%
%  This program is free software; you can redistribute it and/or modify it
%  under the terms of the GNU General Public License as published by the
%  Free Software Foundation (version 2 of the License or  later version).
%
%  This program is distributed WITHOUT ANY WARRANTY.
%  See the GNU General Public License for more details.
%==========================================================================
%	REFERENCES
%==========================================================================
%
%   D. T. Hristopulos, Spartan Gibbs random field models for 
%   geostatistical applications, SIAM Journal of Scientific Computing,
%   24 (6), pp. 2125-2162 (2003).
%
%   D.T. Hristopulos (2005). Spartan Gaussian random fields for 
%   geostatistical applications: Non-constrained simulations on square 
%   lattices and irregular grids.  Journal of Computational Methods 
%   in Science and Engineering 5, 149  164
%
%   D. T. Hristopulos and S. Elogne, Analytic properties and covariance 
%   functions for a new class of generalized Gibbs random fields, 
%   IEEE Transactions on Information Theory, 53(12), 4467-4679 (2007).
%
%   D. T. Hristopulos, Flexible Covariance Functions Related to Spatial 
%   Random Field Models with Local Interactions, 2013, ArXiv: 
%
%==========================================================================
function [x,y,rf]=spartan_sim_v1(Lx,Ly,Param,sd,ifig)

eta0 = Param(1);
eta1 = Param(2);
xi = Param(3);

if length(Param)==4
    km = Param(4); 
end

if (eta1 < -2)
    error('Non-permissible coefficient eta1');
end;

if (2*round(Lx/2)-Lx ~= 0); Lx=Lx+1; 
    display('I increased number of nodes by 1'); end;
if (2*round(Ly/2)-Ly ~= 0); Ly=Ly+1; 
    display('I increased number of nodes by 1'); end;

nx=1:Lx;
ny=1:Ly;

[x,y] = meshgrid(nx,ny);

fx=1:Lx;
fy=1:Ly;

%   The frequency counting is organized so that
%   the spectrum of X(q) satisfies the necessary
%   relations for the IFFT to generate a real field.

knx = 2*pi*(fx-Lx/2-1)/Lx;
kny = 2*pi*(fy-Ly/2-1)/Ly;

[kx,ky]=meshgrid(knx,kny);

tstring='Spartan Gibbs RF density';
ksq = xi^2 *( kx.^2+ky.^2 );
Sq = eta0 * (xi^2)./ ( 1 + eta1*ksq + ksq.^2 );

if (exist('km','var') && ~isempty(km) )
    Sq = Sq .*( ksq < (km*xi)^2) ;
end

%	The spectrum needs to be reorganized
%	before the IFFT2 operation so that the
%   low frequencies are near the left corner

Ssq=fftshift(Sq);

if exist('sd','var') && ~isempty(sd)
    rng(sd);
else
    rng(sum(100*clock));
end
%	The new algorithm for calculating the
%	spectral component of the random numbers
%   The real part of the IFFT is taken, because
%   the IFFT contains complex phases even when the
%   FFT X(k) is Hermitian, due to the fact that the
%   MATLAB routine uses a non-symmetric frequency sequence
%
% --------------------------------------------
%   The matrix U contains the (real) random components

u = zeros(Ly,Lx);
nlx = Lx/2+1;
nly = Ly/2+1;

u1 = randn(nly,nlx) + sqrt(-1)*randn(nly,nlx);
u1(1,1) = real(u1(1,1));
u1(1,nlx) = real(u1(1,nlx));
u1(nly,1) = real(u1(nly,1));
u1(nly,nlx) = real(u1(nly,nlx));
u(1:nly,1:nlx) = u1;

for i=nlx+1:Lx
    u(1,i) = conj(u1(1,Lx+2-i));
    u(nly,i) = conj(u1(nly,Lx+2-i));
end;

for i=nly+1:Ly
    u(i,1) = conj(u1(Ly+2-i,1));
    u(i,nlx) = conj(u1(Ly+2-i,nlx));
end;

u1part = u1(2:nly-1,2:nlx-1);
u2 = randn(nly-2,nlx-2) + sqrt(-1)*randn(nly-2,nlx-2);
u(nly+1:Ly,nlx+1:Lx) = conj(rot90(u1part,2));
u(nly+1:Ly,2:nlx-1) = u2;
u(2:nly-1,nlx+1:Lx) = conj(rot90(u2,2));

%   We normalize u by the sqrt(2) so that the 
%   random variables u have magnitude equal to one
rfq = sqrt(Ssq) .* u /sqrt(2);

% The normalization factor transforms from the continuum spectral density to
% the discrete spectral density
rf = ifft2(rfq)*sqrt(Lx*Ly);

%   This is necessary because for large domains there is
%   a negligibly small complex part due to numerical errors
%   of the IFFT
rf=real(rf);

srf=std(reshape(rf, [Ly*Lx 1]));
display(['Standard Deviation of Gibbs SSRF=',num2str(srf)]);

if exist('ifig','var') && ~isempty(ifig)
    
    figure(1); 
    contourf(Ssq); 
    colorbar; 
    shading flat;
    title(tstring);
    
    figure(2); 
    contour(kx,ky,Sq); 
    colorbar;
    axis tight;
    title('SSRF Spectral Density (First Brillouin Zone)');

    figure(3);
    surf(x,y,rf);
    colorbar;
    shading flat;
    title('SSRF'); 
end;
